@@ -1,20 +1,17 @@
 import atexit
-import logging
 import gc
+import logging
 import os
 import re
 import sys
 import warnings
 
 import click
-import dask
-
 from tornado.ioloop import IOLoop
 
-from distributed import Scheduler, Security
-from distributed.preloading import validate_preload_argv
+from distributed import Scheduler
 from distributed.cli.utils import check_python_3, install_signal_handlers
-from distributed.utils import deserialize_for_cli
+from distributed.preloading import validate_preload_argv
 from distributed.proctitle import (
     enable_proctitle_on_children,
     enable_proctitle_on_current,
@@ -106,7 +103,6 @@ pem_file_option_type = click.Path(exists=True, resolve_path=True)
     type=str,
     multiple=True,
     is_eager=True,
-    default="",
     help="Module that should be loaded by the scheduler process  "
     'like "foo.bar" or "/path/to/foo.py".',
 )
@@ -157,22 +153,15 @@ def main(
     if port is None and (not host or not re.search(r":\d", host)):
         port = 8786
 
-    sec = Security(
-        **{
-            k: v
-            for k, v in [
-                ("tls_ca_file", tls_ca_file),
-                ("tls_scheduler_cert", tls_cert),
-                ("tls_scheduler_key", tls_key),
-            ]
-            if v is not None
-        }
-    )
-
-    if "DASK_INTERNAL_INHERIT_CONFIG" in os.environ:
-        config = deserialize_for_cli(os.environ["DASK_INTERNAL_INHERIT_CONFIG"])
-        # Update the global config given priority to the existing global config
-        dask.config.update(dask.config.global_config, config, priority="old")
+    sec = {
+        k: v
+        for k, v in [
+            ("tls_ca_file", tls_ca_file),
+            ("tls_scheduler_cert", tls_cert),
+            ("tls_scheduler_key", tls_key),
+        ]
+        if v is not None
+    }
 
     if not host and (tls_ca_file or tls_cert or tls_key):
         host = "tls://"
