@@ -11,23 +11,24 @@ except ImportError:
 from .components.nvml import gpu_doc  # noqa: 1708
 from .components.nvml import NVML_ENABLED, gpu_memory_doc, gpu_utilization_doc
 from .components.scheduler import (
+    AggregateAction,
+    BandwidthTypes,
+    BandwidthWorkers,
+    ComputePerKey,
+    CurrentLoad,
+    MemoryByKey,
+    NBytes,
+    NBytesCluster,
+    SystemMonitor,
+    TaskGraph,
+    TaskProgress,
+    TaskStream,
+    WorkerTable,
     events_doc,
     graph_doc,
-    individual_aggregate_time_per_action_doc,
-    individual_bandwidth_types_doc,
-    individual_bandwidth_workers_doc,
-    individual_compute_time_per_key_doc,
-    individual_cpu_doc,
-    individual_graph_doc,
-    individual_memory_by_key_doc,
-    individual_nbytes_doc,
-    individual_nprocessing_doc,
+    individual_doc,
     individual_profile_doc,
     individual_profile_server_doc,
-    individual_progress_doc,
-    individual_systemmonitor_doc,
-    individual_task_stream_doc,
-    individual_workers_doc,
     profile_doc,
     profile_server_doc,
     status_doc,
@@ -36,7 +37,6 @@ from .components.scheduler import (
     tasks_doc,
     workers_doc,
 )
-from .components.worker import counters_doc
 from .core import BokehApplication
 from .worker import counters_doc
 
@@ -79,21 +79,26 @@ applications = {
     "/profile-server": profile_server_doc,
     "/graph": graph_doc,
     "/gpu": gpu_doc,
-    "/individual-task-stream": individual_task_stream_doc,
-    "/individual-progress": individual_progress_doc,
-    "/individual-graph": individual_graph_doc,
+    "/individual-task-stream": individual_doc(
+        TaskStream, 100, n_rectangles=1000, clear_interval="10s"
+    ),
+    "/individual-progress": individual_doc(TaskProgress, 100, height=160),
+    "/individual-graph": individual_doc(TaskGraph, 200),
+    "/individual-nbytes": individual_doc(NBytes, 100),
+    "/individual-nbytes-cluster": individual_doc(NBytesCluster, 100),
+    "/individual-cpu": individual_doc(CurrentLoad, 100, fig_attr="cpu_figure"),
+    "/individual-nprocessing": individual_doc(
+        CurrentLoad, 100, fig_attr="processing_figure"
+    ),
+    "/individual-workers": individual_doc(WorkerTable, 500),
+    "/individual-bandwidth-types": individual_doc(BandwidthTypes, 500),
+    "/individual-bandwidth-workers": individual_doc(BandwidthWorkers, 500),
+    "/individual-memory-by-key": individual_doc(MemoryByKey, 500),
+    "/individual-compute-time-per-key": individual_doc(ComputePerKey, 500),
+    "/individual-aggregate-time-per-action": individual_doc(AggregateAction, 500),
+    "/individual-scheduler-system": individual_doc(SystemMonitor, 500),
     "/individual-profile": individual_profile_doc,
     "/individual-profile-server": individual_profile_server_doc,
-    "/individual-nbytes": individual_nbytes_doc,
-    "/individual-cpu": individual_cpu_doc,
-    "/individual-nprocessing": individual_nprocessing_doc,
-    "/individual-workers": individual_workers_doc,
-    "/individual-bandwidth-types": individual_bandwidth_types_doc,
-    "/individual-bandwidth-workers": individual_bandwidth_workers_doc,
-    "/individual-memory-by-key": individual_memory_by_key_doc,
-    "/individual-compute-time-per-key": individual_compute_time_per_key_doc,
-    "/individual-aggregate-time-per-action": individual_aggregate_time_per_action_doc,
     "/individual-gpu-memory": gpu_memory_doc,
     "/individual-gpu-utilization": gpu_utilization_doc,
-    "/individual-scheduler-system": individual_systemmonitor_doc,
 }
