@@ -2,10 +2,28 @@ import pytest
 
 pytest.importorskip("asyncssh")
 
+import sys
+
 import dask
-from dask.distributed import Client
+
+from distributed import Client
+from distributed.compatibility import MACOS, WINDOWS
 from distributed.deploy.ssh import SSHCluster
-from distributed.utils_test import loop  # noqa: F401
+
+pytestmark = [
+    pytest.mark.xfail(MACOS, reason="very high flakiness; see distributed/issues/4543"),
+    pytest.mark.skipif(WINDOWS, reason="no CI support; see distributed/issues/4509"),
+]
+
+
+def test_ssh_hosts_None():
+    with pytest.raises(ValueError):
+        SSHCluster(hosts=None)
+
+
+def test_ssh_hosts_empty_list():
+    with pytest.raises(ValueError):
+        SSHCluster(hosts=[])
 
 
 @pytest.mark.asyncio
@@ -50,7 +68,7 @@ async def test_keywords():
             assert all(v["nthreads"] == 2 for v in d.values())
 
 
-@pytest.mark.avoid_travis
+@pytest.mark.avoid_ci
 def test_defer_to_old(loop):
     with pytest.warns(Warning):
         with SSHCluster(
@@ -63,8 +81,8 @@ def test_defer_to_old(loop):
             assert isinstance(c, OldSSHCluster)
 
 
-@pytest.mark.avoid_travis
-def test_old_ssh_wih_local_dir(loop):
+@pytest.mark.avoid_ci
+def test_old_ssh_with_local_dir(loop):
     with pytest.warns(Warning):
         from distributed.deploy.old_ssh import SSHCluster as OldSSHCluster
 
@@ -146,3 +164,43 @@ async def test_list_of_connect_options_raises():
             worker_options={"death_timeout": "5s"},
         ) as _:
             pass
+
+
+@pytest.mark.asyncio
+async def test_remote_python():
+    async with SSHCluster(
+        ["127.0.0.1"] * 3,
+        connect_options=[dict(known_hosts=None)] * 3,
+        asynchronous=True,
+        scheduler_options={"port": 0, "idle_timeout": "5s"},
+        worker_options={"death_timeout": "5s"},
+        remote_python=sys.executable,
+    ) as cluster:
+        assert cluster.workers[0].remote_python == sys.executable
+
+
+@pytest.mark.asyncio
+async def test_remote_python_as_dict():
+    async with SSHCluster(
+        ["127.0.0.1"] * 3,
+        connect_options=[dict(known_hosts=None)] * 3,
+        asynchronous=True,
+        scheduler_options={"port": 0, "idle_timeout": "5s"},
+        worker_options={"death_timeout": "5s"},
+        remote_python=[sys.executable] * 3,
+    ) as cluster:
+        assert cluster.workers[0].remote_python == sys.executable
+
+
+@pytest.mark.asyncio
+async def test_list_of_remote_python_raises():
+    with pytest.raises(RuntimeError):
+        async with SSHCluster(
+            ["127.0.0.1"] * 3,
+            connect_options=[dict(known_hosts=None)] * 3,
+            asynchronous=True,
+            scheduler_options={"port": 0, "idle_timeout": "5s"},
+            worker_options={"death_timeout": "5s"},
+            remote_python=[sys.executable] * 4,  # Mismatch in length 4 != 3
+        ) as _:
+            pass
