@@ -5,14 +5,7 @@ import pytest
 from distributed import Scheduler, Worker
 from distributed.diagnostics.progressbar import TextProgressBar, progress
 from distributed.metrics import time
-from distributed.utils_test import (  # noqa: F401
-    client,
-    cluster_fixture,
-    div,
-    gen_cluster,
-    inc,
-    loop,
-)
+from distributed.utils_test import div, gen_cluster, inc
 
 
 def test_text_progressbar(capsys, client):
@@ -49,8 +42,8 @@ async def test_TextProgressBar_error(c, s, a, b):
 @pytest.mark.asyncio
 async def test_TextProgressBar_empty(capsys):
     async with Scheduler(port=0) as s:
-        async with Worker(s.address, nthreads=1) as a:
-            async with Worker(s.address, nthreads=1) as b:
+        async with Worker(s.address, nthreads=1):
+            async with Worker(s.address, nthreads=1):
                 progress = TextProgressBar(
                     [], scheduler=s.address, start=False, interval=0.01
                 )
